/* Module dependencies */

var cps = require('cps-api'),
	shortid = require('shortid');

exports.initialize = function initializeSchema(schema, callback) {
	
	if (!cps) {
        throw new Error('module cps-api is not defined, try\n  npm install cps-api');
    }
	var s = schema.settings;

	var conSettings = {
        host: s.host,
        storage: s.storage,
        user: s.user,
        password: s.password,
        accountid: s.accountid
    };
    
	schema.cps = cps;
	schema.client = new schema.cps.Connection(s.host,s.storage,s.user,s.password, 'document', '//document/id', {account: s.account});
	schema.adapter = new Clusterpoint(schema.cps,schema.client);
	schema.adapter.schema = schema;	
	
}

function Clusterpoint(cps,client) {
    this._models = {};
    this.cps = cps;
    this.client = client;
    
}

Clusterpoint.prototype.define = function(descr) {

    this._models[descr.model.modelName] = descr;
    this.schema.connected = true;
    
};

Clusterpoint.prototype.defineProperty = function(model, prop, params) {
	
    this._models[model].properties[prop] = params;
    
};

Clusterpoint.prototype.create = function(model, data, callback) {
	
	data.id = shortid.generate();
	data._model = model;
	
	this.client.sendRequest(new cps.UpdateRequest(filterQuery(data)), function (err, response) {
		
		if (err) return callback(err);
		
		callback(err,response.document.id);
		
	});
	
};

Clusterpoint.prototype.updateOrCreate = function(model, data, callback) {
	
	data._model = model;
	
	this.client.sendRequest(new cps.UpdateRequest(filterQuery(data)), function (err, response) {
		
		if (err) return callback(err);
		
		callback(err,response.document.id);
		
	});	
	
};

Clusterpoint.prototype.save = function(model, data, callback) {
	
	data._model = model;
	
	this.client.sendRequest(new cps.UpdateRequest(filterQuery(data)), function (err) {
		
		callback(err);
		
	});

};

Clusterpoint.prototype.exists = function exists(model, id, callback) {

	var query = this.cps.ObjectTerm({
			'_model': model,
			'id': id
		});
		
	var request = new cps.SearchRequest(query);
    
    this.client.sendRequest(request, function (err, response) {
		
		callback(err, !err && response.found!=0);
		
	});

};

Clusterpoint.prototype.find = function findById(model, id, callback) {

	var query = this.cps.ObjectTerm({
			'_model': model,
			'id': id
		});
		
	var request = new cps.SearchRequest(query);
    
    this.client.sendRequest(request, function (err, response) {
		
		if (err) return callback(err);
		
		callback(err,(response.found!=0) ?  filterResponse(response.results.document) : false);
		
	});

};

Clusterpoint.prototype.destroy = function destroy(model, id, callback) {
	
	var query = this.cps.ObjectTerm({
			'document/id':id
		});
	
    var request = new cps.DeleteRequest(query);
    
    this.client.sendRequest(request,callback);
    
};

Clusterpoint.prototype.remove = function remove(model, filter, callback) {

	
	var query = filter.where;
		query._model = model;
	
	var request = new cps.SearchRequest(this.cps.ObjectTerm(query));
	
	/* TODO: when Clusterpoint updated itself to 2.3 */
};

Clusterpoint.prototype.all = function all(model, filter, callback) {
   
	var query = (filter.where) ? filter.where : {},
		limit = 0;
		//limit = (filter.limit) ? filter.limit : null;
	
		query._model = model;

	var request = new cps.SearchRequest(this.cps.ObjectTerm(query),limit);
	
	// TODO: order

	this.client.sendRequest(request, function (err, response) {
		
		if (err) return callback(err);
		
		callback(null,(response.found!=0) ? filterResponse(response.results.document) : false);
		
	});	
   
};

Clusterpoint.prototype.destroyAll = function destroyAll(model, callback) {
    
    /* TDOD: when Clusterpoint updated itself to 2.3 */
    
};

Clusterpoint.prototype.count = function count(model, callback, filter) {
    
    var query = (filter.where) ? filter.where : {},
		limit = (filter.limit) ? filter.limit : 0;
	
		query._model = model;

	var request = new cps.SearchRequest(this.cps.ObjectTerm(query),limit);
	
	this.client.sendRequest(request, function (err, response) {
		
		if (err) return callback(err);
		
		callback(null,(response.found!=0) ? response.results.document.length : 0);
		
	});	
     
};

Clusterpoint.prototype.updateAttributes = function (model, id, data, callback) {

	data._model = model;
	data.id = id;
	
	this.client.sendRequest(new cps.UpdateRequest(data), function (err) {
		
		callback(err);
		
	});

};

function filterResponse(response,root){
	
	var model = '';
	
	if (typeof root === 'undefined') { root =  true;};
	
	for(var key in response){
		
		if(typeof response[key] === 'object'){
			response[key] = filterResponse(response[key],false);
		}
		if (key == '_model') {
			model = response[key]; 
			delete response[key];
		}
	}
    
    return (!(response instanceof Array) && root) ? Array(response) : response;

}

function filterQuery(query){
	
	
	for(var key in query){
		if(typeof query[key] === "undefined"){
			delete query[key];
		}
	}
    
    return query;

}